import axios from 'axios'
/* resourcePaths*/
import {
    INIT_PAYMENTS_PURCHASES_URL,
    PAYMENTS_PURCHASES_PROCESS_URL
} from '@/constants/resourcePaths'
/* Actions */
import {
    INIT_PAYMENTS_PURCHASES,
    PAYMENTS_PURCHASES_NOTIFY,
    EXPLODE_DATA,
    PAYMENTS_PURCHASE_PROCESS,
    COMPLETED_PAYMENTS_PURCHASES,
    RESET_PAYMENTS_PURCHASES,
    CHECK_PAYMENTS_PURCHASES_SUPPLIER,
    CHECK_PAYMENTS_PURCHASES_INVOICES,
    CHECK_PAYMENTS_PURCHASES_LINES_PAYMENTS,
    CHECK_PAYMENTS_PURCHASES_JOURNALS,
    SET_CHANGE_TAB_STEPS,
    CHANGE_INITIAL_PAID,
    ACTION_REMOVE_ALL_SUPPLIER_LINES_PAYMENTS
} from '@/constants/actionTypes'
/* Mutations */
import {
    SET_COMPLETED_PAYMENTS_PURCHASES,
    SET_PROCESSING_PAYMENTS_PURCHASES
} from '@/constants/mutationTypes'

const actions = {
    /**
     * [PAYMENTS_PURCHASES_NOTIFY]
     */
    [PAYMENTS_PURCHASES_NOTIFY] ({ commit }, paylod) {
        openerp.web.notification.do_warn('Atencion', paylod)
        return false
    },
    /**
     * [response description]
     * @type {[type]}
     */
    [INIT_PAYMENTS_PURCHASES] ({ dispatch }) {
        return axios.get(INIT_PAYMENTS_PURCHASES_URL).then(response => {
            dispatch(COMPLETED_PAYMENTS_PURCHASES, false)
            dispatch(EXPLODE_DATA, response.data)
        }).catch(error => {
            console.log(error);
        } )
    },
    /**
     *  [EXPLODE_DATA]
     */
    [EXPLODE_DATA] ({ dispatch },paylod) {
        for (let value in paylod) {
            dispatch(`initPaymentsPurchases${value[0].toUpperCase()}${value.slice(1)}`, paylod[value])
        }
    },
    /**
     * [PAYMENTS_PURCHASE_PROCESS]
     */
    [PAYMENTS_PURCHASE_PROCESS] ({ commit, dispatch, getters }){
        commit(SET_PROCESSING_PAYMENTS_PURCHASES, true)
        const data = {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                journalId: getters.paymentsSelectedJournal.id,
                amountPayments: getters.paidTotal <= getters.paymentsTotal ? getters.paidTotal : getters.paymentsTotal,
                supplierId: getters.selectedSupplier.id,
                invoiceId: getters.selectedInvoices.id,
                moveLinesId: getters.paymentsMoveLines.map( item => {
                    return item.id
                })
            }
        }

        return axios.post(PAYMENTS_PURCHASES_PROCESS_URL, data).then(response => {
            commit(SET_PROCESSING_PAYMENTS_PURCHASES, !response)
            dispatch(COMPLETED_PAYMENTS_PURCHASES, response.data.result)
        }).catch(error => {
            commit(SET_PROCESSING_PAYMENTS_PURCHASES, !false)
        })
    },
    /**
     * [COMPLETED_PAYMENTS_PURCHASES] [Cambiar el estado cunado se completa el pago]
     */
    [COMPLETED_PAYMENTS_PURCHASES] ({ commit }, payload ) {
        commit(SET_COMPLETED_PAYMENTS_PURCHASES, !!payload)
    },
    /**
     *  [RESET_PAYMENTS_PURCHASES]
     */
    [RESET_PAYMENTS_PURCHASES] ({ rootState, commit, dispatch }) {
        for (let key in rootState ) {
            if (!(rootState[key] instanceof Object)){
                continue
            }
            dispatch(`reset${key[0].toUpperCase()}${key.slice(1)}`)
        }
        dispatch(INIT_PAYMENTS_PURCHASES)
    },
    /* CHECH STEPS*/
    /**
     * [CHECK_PAYMENTS_PURCHASES_SUPPLIER] [Validar que seleccione un proveedor]
     */
     [CHECK_PAYMENTS_PURCHASES_SUPPLIER] ({ dispatch, getters }) {
         return !!getters.selectedSupplier || dispatch(PAYMENTS_PURCHASES_NOTIFY, 'Necesitas seleccionar un proveedor para continuar.')
     },
     /**
      * [CHECK_PAYMENTS_PURCHASES_INVOICES] [validar que selecciones una factura]
      */
     [CHECK_PAYMENTS_PURCHASES_INVOICES] ({ dispatch, getters}) {
         return !!getters.selectedInvoices || dispatch(PAYMENTS_PURCHASES_NOTIFY, 'Necesitas seleccionar una factura para continuar.')
     },
     /**
      * [CHECK_PAYMENTS_PURCHASES_LINES_PAYMENTS] [validar que seleccione al menos una cuota a apagar]
      */
     [CHECK_PAYMENTS_PURCHASES_LINES_PAYMENTS] ({ dispatch, getters }) {
         return !!getters.supplierLinesPayments.length || dispatch(PAYMENTS_PURCHASES_NOTIFY, 'Necesitas seleccionar al menos una cuota para continuar.')
     },
     /**
      * [CHECK_PAYMENTS_PURCHASES_JOURNALS] [validar que seleccione el método de pago, ingrese el monto a pagar]
      */
     [CHECK_PAYMENTS_PURCHASES_JOURNALS] ({ dispatch, getters }) {
         if (getters.paymentsSelectedJournal.length <= 0) {
              return dispatch(PAYMENTS_PURCHASES_NOTIFY, 'Necesitas seleccionar el método para continuar.')
         }
         if (getters.paidTotal <= 0){
             return dispatch(PAYMENTS_PURCHASES_NOTIFY, 'El monto no puede ser cero(0), ingrese un monto para continuar.')
         }
         return true
     },
     /**
      * [prev description] [Cuando pasa de un Estado Superior a inferior u cuando abandona el modulo restaura los datos]
      * @type {[type]}
      */
     [SET_CHANGE_TAB_STEPS] ({ commit, dispatch, getters }, paylod) {
         console.log(paylod);
         /* Si abandona el modulo */
         if (paylod.prev === 0 && paylod.next === -1){
            dispatch(RESET_PAYMENTS_PURCHASES)
         }

         if (paylod.prev >= 2 && paylod.next<= 1) {
             dispatch(CHANGE_INITIAL_PAID, 0)
             dispatch(ACTION_REMOVE_ALL_SUPPLIER_LINES_PAYMENTS)
         }
     }
}

export default actions
