/* Actions */
import {
    INIT_SUPPLIER_INVOICES,
    FILTER_SUPPLIER_INVOICES,
    SELECT_SUPPLIER_INVOICES,
    INIT_SUPPLIER_MOVE_LINES,
    ADD_MOVE_IN_INVOICE,
    RESET_SUPPLIER_INVOICES
} from '@/constants/actionTypes'
/* Mutations */
import {
    SET_SUPPLIER_INVOICES,
    SET_SELECTED_SUPPLIER_INVOICES,
    SET_FILTER_SUPPLIER_INVOICES,
    SET_MOVE_IN_INVOICE,
    SET_CURRENCY_INVOICE
} from '@/constants/mutationTypes'

const initialState = {
    supplierInvoices: [],
    filterInvoices: [],
    selectedInvoices: null,
    currencyInvoice :[]
}
const state = {
    supplierInvoices: initialState.supplierInvoices,
    filterInvoices: initialState.filterInvoices,
    selectedInvoices: initialState.selectedInvoices,
    currencyInvoice: initialState.currencyInvoice
}
const getters = {
    /**
     * [supplierInvoices description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    supplierInvoices (state) {
        return state.supplierInvoices
    },
    /**
     * [filterInvoices description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    supplierInvoicesVisibles ( state ) {
        return state.filterInvoices.length ===0 ? state.supplierInvoices :  state.filterInvoices
    },
    /**
     * [selectedInvoices description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    selectedInvoices ( state ) {
        return state.selectedInvoices
    },
    /**
     * [currencyInvoice description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    currencyInvoice ( state ) {
        return state.currencyInvoice
    }
}

const mutations = {
    /**
     * [Invoices description]
     * @type {[type]}
     */
    [SET_SUPPLIER_INVOICES] (state, payload ) {
            state.supplierInvoices = payload
    },
    /**
     * [selectedInvoices description]
     * @type {[type]}
     */
    [SET_SELECTED_SUPPLIER_INVOICES] ( state, payload ) {
        state.selectedInvoices= payload
    },
    /**
     * [filterInvoices description]
     * @type {[type]}
     */
    [SET_FILTER_SUPPLIER_INVOICES] ( state, payload ) {
        state.filterInvoices = payload
    },
    /**
     * [mode description]
     * @type {[type]}
     */
    [SET_MOVE_IN_INVOICE] (state, payload) {
        if (payload.mode === 'full') {
            payload.moveLines.forEach(moves => {
                let movesFound = state.selectedInvoices.moveLines.find(item => item.id === moves.id)
                if (movesFound) return
                state.selectedInvoices.moveLines = [moves, ...state.selectedInvoices.moveLines]
            })
        } else {
            let movesFound = state.selectedInvoices.moveLines.find(item => item.id === payload.moveLines.id)
            if (movesFound) return
            state.selectedInvoices.moveLines = [payload.moveLines, ...state.selectedInvoices.moveLines]
        }
    },
    [SET_CURRENCY_INVOICE] ( state, payload) {
        state.currencyInvoice = payload
    }
}
const actions = {
    /**
     * [INIT_SUPPLIER_INVOICES]
     */
    [INIT_SUPPLIER_INVOICES] ({ commit }, payload ) {
        commit(SET_SUPPLIER_INVOICES, payload)
    },
    /**
     * [FILTER_SUPPLIER_INVOICES]
     */
    [FILTER_SUPPLIER_INVOICES] ({ commit }, payload ) {
        commit(SET_FILTER_SUPPLIER_INVOICES, payload)
    },
    /**
     * [SELECT_SUPPLIER_INVOICES]
     */
    [SELECT_SUPPLIER_INVOICES] ({commit, dispatch}, payload) {
        commit(SET_SELECTED_SUPPLIER_INVOICES, payload)
        commit(SET_CURRENCY_INVOICE, payload.currencyInvoice)

        dispatch(INIT_SUPPLIER_MOVE_LINES, payload.moveLines)
    },
    /**
     * [ADD_MOVE_IN_INVOICE]
     */
    [ADD_MOVE_IN_INVOICE] ({ commit },payload ) {
        commit(SET_MOVE_IN_INVOICE, payload)
    },
    /**
     * [RESET_SUPPLIER_INVOICES]
     */
    [RESET_SUPPLIER_INVOICES] ({ commit }) {
        commit(SET_SUPPLIER_INVOICES, [])
        commit(SET_SELECTED_SUPPLIER_INVOICES, null)
        commit(SET_FILTER_SUPPLIER_INVOICES, [])
        commit(SET_CURRENCY_INVOICE, [])
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}
