/* Actions */
import {
    INIT_SUPPLIER_MOVE_LINES,
    SELECT_SUPPLIER_MOVE_LINES,
    ACTION_REMOVE_SUPPLIER_LINES_PAYMENTS,
    ADD_MOVE_IN_INVOICE,
    ADD_PAYMENTS_MOVE_LINES,
    REMOVE_PAYMENTS_MOVE_LINES,
    RESET_SUPPLIER_MOVE_LINES,
    ACTION_REMOVE_ALL_SUPPLIER_LINES_PAYMENTS,
    RESET_SELECTED_JOURNAL_PAYMENTS
} from '@/constants/actionTypes'
/* Mutations */
import {
    SET_SUPPLIER_MOVE_LINES,
    SET_SELECTED_SUPPLIER_MOVE_LINES,
    SET_SUPPLIER_LINES_PAYMENTS,
    REMOVE_SUPPLIER_MOVE_LINES,
    SET_ORDER_SUPPLIER_MOVE_LINES,
    SET_ORDER_SUPPLIER_LINES_PAYMENTS,
    SET_CALCULATE_TOTAL,
    REMOVE_SUPPLIER_LINES_PAYMENTS,
    REMOVE_LINES_PAYMENTS
} from '@/constants/mutationTypes'

const intialState = {
    supplierMoveLines: [],
    selectdLines: [],
    supplierLinesPayments: [],
    supplierPaymentsTotal: 0
}
const  state = {
    supplierMoveLines: intialState.supplierMoveLines,
    selectdLines: intialState.selectdLines,
    supplierLinesPayments: intialState.supplierLinesPayments,
    supplierPaymentsTotal: intialState.supplierPaymentsTotal
}
const getters = {
    /**
     * [supplierMoveLines description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    supplierMoveLines ( state ) {
        return state.supplierMoveLines
    },
    /**
     * [selectdLines description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    selectdLines ( state ){
        return state.selectdLines
    },
    /**
     * [supplierLinesPayments description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    supplierLinesPayments ( state ) {
        return state.supplierLinesPayments
    },
    supplierPaymentsTotal ( state ) {
        return state.supplierPaymentsTotal
    }
}
const mutations = {
    /**
     * [supplierInvoices description]
     * @type {[type]}
     */
    [SET_SUPPLIER_MOVE_LINES] (state, payload) {
        state.supplierMoveLines = payload
    },
    /**
     * [selectdLines description]
     * @type {[type]}
     */
    [SET_SELECTED_SUPPLIER_MOVE_LINES] ( state, payload) {
        state.selectdLines = payload
    },
    /**
     * [supplierLinesPayments description]
     * @type {Array}
     */
    [SET_SUPPLIER_LINES_PAYMENTS] (state, payload) {
        state.supplierLinesPayments = [payload, ...state.supplierLinesPayments]
    },
    /**
     * [amountTot description]
     * @type {[type]}
     */
    [REMOVE_SUPPLIER_MOVE_LINES] ( state, payload ) {
        let moveFoundIndex = state.supplierMoveLines.findIndex(item => item.id === payload.id)
        state.supplierMoveLines.splice(moveFoundIndex, 1)
    },
    /**
     * [mode description]
     * @type {[type]}
     */
    [REMOVE_SUPPLIER_LINES_PAYMENTS] (state, payload ) {
        if (payload.mode === 'full') {
            payload.moveLines.forEach(moves => {
                //ADD supplierMoveLines
                let movesFound = state.supplierMoveLines.find(item => item.id === moves.id)
                if (movesFound) return
                state.supplierMoveLines = [moves, ...state.supplierMoveLines]
                // Remove Payments
                let movesPaymentsIndex = state.supplierLinesPayments.findIndex(item => item.id === moves.id)
                state.supplierLinesPayments.splice(movesPaymentsIndex)
            })
        } else {
            //ADD supplierMoveLines
            let movesFound = state.supplierMoveLines.find(item => item.id === payload.moveLines.id)
            if (movesFound) return
            state.supplierMoveLines = [payload.moveLines, ...state.supplierMoveLines]
            // Remove Payments
            let movesPaymentsIndex = state.supplierLinesPayments.findIndex(item => item.id === payload.moveLines.id)
            state.supplierLinesPayments.splice(movesPaymentsIndex, 1)
        }
    },
    /**
     * [mode description]
     * @type {[type]}
     */
    [REMOVE_LINES_PAYMENTS] (state, payload ) {
        if (payload.mode === 'full') {
            payload.moveLines.forEach(moves => {
                // Remove Payments
                let movesPaymentsIndex = state.supplierLinesPayments.findIndex(item => item.id === moves.id)
                state.supplierLinesPayments.splice(movesPaymentsIndex)
            })
        } else {
            // Remove Payments
            let movesPaymentsIndex = state.supplierLinesPayments.findIndex(item => item.id === payload.moveLines.id)
            state.supplierLinesPayments.splice(movesPaymentsIndex, 1)
        }
    },
    /**
     * [amountTot description]
     * @type {[type]}
     */
    [SET_CALCULATE_TOTAL] ( state ) {
        let amountTot = 0
        if (state.supplierLinesPayments.length > 0)
            state.supplierLinesPayments.forEach( item => {
                amountTot = amountTot + item.amountResidualCurrency
            })

        state.supplierPaymentsTotal =  amountTot
    },
    /**
     * [supplierMoveLines description]
     * @type {[type]}
     */
    [SET_ORDER_SUPPLIER_MOVE_LINES] ( state ) {
        state.supplierMoveLines =  state.supplierMoveLines.sort((a, b) => {
            return a.dateMaturity > b.dateMaturity
        })
    },
    /**
     * [supplierLinesPayments description]
     * @type {[type]}
     */
    [SET_ORDER_SUPPLIER_LINES_PAYMENTS]( state ) {
        state.supplierLinesPayments =  state.supplierLinesPayments.sort((a, b) => {
            return a.dateMaturity > b.dateMaturity
        })
    }
}

const actions = {
    /**
     * [INIT_SUPPLIER_MOVE_LINES]
     */
    [INIT_SUPPLIER_MOVE_LINES] ({ commit }, payload) {
        commit(SET_SUPPLIER_MOVE_LINES, payload)
        commit(SET_ORDER_SUPPLIER_MOVE_LINES)
    },
    /**
     *  [SELECT_SUPPLIER_MOVE_LINES]
     */
    [SELECT_SUPPLIER_MOVE_LINES] ({ commit, dispatch }, payload) {
        // payments
        commit(SET_SUPPLIER_LINES_PAYMENTS, payload)
        commit(SET_ORDER_SUPPLIER_LINES_PAYMENTS)
        commit(SET_SELECTED_SUPPLIER_MOVE_LINES, payload)
        commit(REMOVE_SUPPLIER_MOVE_LINES, payload)
        commit(SET_CALCULATE_TOTAL)
        dispatch(ADD_PAYMENTS_MOVE_LINES, payload)
    },
    /**
     * [mode description]
     * @type {String}
     */
    [ACTION_REMOVE_SUPPLIER_LINES_PAYMENTS] ( {commit, dispatch}, payload ) {
        dispatch(ADD_MOVE_IN_INVOICE, {
            mode: 'partial',
            moveLines: payload
        })
        commit(REMOVE_SUPPLIER_LINES_PAYMENTS, {
            mode: 'partial',
            moveLines: payload
        })
        dispatch(REMOVE_PAYMENTS_MOVE_LINES, {
            mode: 'partial',
            moveLines: payload
        })
        // Total
        commit(SET_CALCULATE_TOTAL)
        commit(SET_ORDER_SUPPLIER_MOVE_LINES)
    },
    /**
     * [mode description]
     * @type {String}
     */
    [ACTION_REMOVE_ALL_SUPPLIER_LINES_PAYMENTS] ({ commit, dispatch, getters }) {

        dispatch(ADD_MOVE_IN_INVOICE, {
            mode: 'full',
            moveLines: getters.supplierLinesPayments.map(item => {
                return item
            })
        })
        dispatch(REMOVE_PAYMENTS_MOVE_LINES, {
            mode: 'full',
            moveLines: getters.supplierLinesPayments.map(item => {
                return item
            })
        })
        dispatch(RESET_SELECTED_JOURNAL_PAYMENTS)
        commit(REMOVE_SUPPLIER_LINES_PAYMENTS, {
            mode: 'full',
            moveLines: getters.supplierLinesPayments.map(item => {
                return item
            })
        })
        commit(SET_CALCULATE_TOTAL)
        commit(SET_ORDER_SUPPLIER_MOVE_LINES)
    },
    /**
     * [RESET_SUPPLIER_MOVE_LINES]
     */
    [RESET_SUPPLIER_MOVE_LINES] ({ commit, getters }){
        commit(SET_SUPPLIER_MOVE_LINES, [])
        commit(SET_SELECTED_SUPPLIER_MOVE_LINES, [])
        commit(REMOVE_LINES_PAYMENTS, {
                mode: 'full',
                moveLines:getters.supplierLinesPayments
        })
        commit(SET_CALCULATE_TOTAL)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}
