/* Actions */
import {
    INIT_PAYMENTS_JOURNALS,
    SELECT_PAYMENTS_JOURNALS,
    ADD_PAYMENTS_MOVE_LINES,
    REMOVE_PAYMENTS_MOVE_LINES,
    CHANGE_INITIAL_PAID,
    RESET_SUPPLIER_PAYMENTS,
    RESET_SELECTED_JOURNAL_PAYMENTS
} from '@/constants/actionTypes'
/* Mutations */
import {
    SET_PAYMENTS_JOURNALS,
    SELECTED_PAYMENTS_JOURNALS,
    CALCULATE_PAYMENTS_TOTAL,
    CALCULATE_CURRENCY_AMOUNT,
    SET_ADD_PAYMENTS_MOVE_LINES,
    SET_REMOVE_PAYMENTS_MOVE_LINES,
    SET_PAID_TOTAL,
    SET_PAYMENTS_CURRENCY_JOURNAL
} from '@/constants/mutationTypes'

const initialState = {
    paymentsJournals: [],
    paymentsSelectedJournal: null,
    paymentsMoveLines: [],
    paymentsCurrencyAmount: [],
    paymentsCurrencyJournal: null,
    paymentsTotal: 0,
    paidTotal: 0,
    paymentsReturned: 0
}
const state = {
    paymentsJournals: initialState.paymentsJournals,
    paymentsSelectedJournal: initialState.paymentsSelectedJournal,
    paymentsMoveLines: initialState.paymentsMoveLines,
    paymentsCurrencyAmount : initialState.paymentsCurrencyAmount,
    paymentsCurrencyJournal: initialState.paymentsCurrencyJournal,
    paymentsTotal: initialState.paymentsTotal,
    paidTotal: initialState.paidTotal,
    paymentsReturned: initialState.paymentsReturned
}

const getters = {
    /**
     * [paymentsJournals description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsJournals ( state ) {
            return state.paymentsJournals
    },
    /**
     * [paymentsSelectedJournal description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsSelectedJournal ( state ) {
        return state.paymentsSelectedJournal
    },
    /**
     * [paymentsMoveLines description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsMoveLines ( state ) {
        return state.paymentsMoveLines
    },
    /**
     * [paymentsCurrencyAmount description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsCurrencyAmount ( state ) {
        return state.paymentsCurrencyAmount
    },
    /**
     * [paymentsTotal description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsTotal ( state ) {
        return state.paymentsTotal
    },
    /**
     * [paymentsReturned description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsReturned ( state ) {
        return state.paymentsReturned
    },
    paidTotal ( state ) {
        return state.paidTotal
    },
    /**
     * [paymentsCurrencyJournal description]
     * @param  {[type]} state [description]
     * @return {[type]}       [description]
     */
    paymentsCurrencyJournal ( state ) {
        return state.paymentsCurrencyJournal
    }
}

const mutations = {
    /**
     * [paymentsJournals description]
     * @type {[type]}
     */
    [SET_PAYMENTS_JOURNALS] ( state, payload ){
        state.paymentsJournals = payload
    },
    /**
     * [paymentsSelectedJournal description]
     * @type {[type]}
     */
    [SELECTED_PAYMENTS_JOURNALS] ( state, payload ) {
        state.paymentsSelectedJournal =  payload
    },
    /**
     * [paymentsMoveLines description]
     * @type {Array}
     */
    [SET_ADD_PAYMENTS_MOVE_LINES] (state, payload) {
        state.paymentsMoveLines = [payload, ...state.paymentsMoveLines]
    },
    /**
     * [actions description]
     * @type {Object}
     */
    [SET_REMOVE_PAYMENTS_MOVE_LINES] (state, payload) {
        if (payload.mode === 'full') {
            payload.moveLines.forEach(moves => {
                // Remove Payments
                let movesPaymentsIndex = state.paymentsMoveLines.findIndex(item => item.id === moves.id)
                state.paymentsMoveLines.splice(movesPaymentsIndex)
            })
        } else {
            // Remove Payments
            let movesPaymentsIndex = state.paymentsMoveLines.findIndex(item => item.id === payload.moveLines.id)
            state.paymentsMoveLines.splice(movesPaymentsIndex, 1)
        }
    },
    /**
     * [actions description]
     * @type {Object}
     */
    [CALCULATE_CURRENCY_AMOUNT] ( state, payload ) {
        let currency = payload.currency
        let paymentsMoveLines = payload.paymentsMoveLines
        let paymentsCurrencyAmount = []

        currency.forEach(item => {
            let amount = 0
            paymentsMoveLines.forEach( currency => {
                let currencyMoveLine = currency.currencyAmount.find(move => move.id === item.id)
                amount = amount + currencyMoveLine.amountCurencyResidual
            })
            paymentsCurrencyAmount.push({
                id: item.id,
                symbolCurrency: item.symbol,
                amount: amount,
                symbol: ' ',
                position: 'before',
                thousandsSeparator: item.thousandsSeparator,
                decimalSeparator: item.decimalSeparator,
                decimalPlaces: item.decimalPlaces,
                rateSilent: item.rateSilent
            })
        })
        state.paymentsCurrencyAmount = paymentsCurrencyAmount
    },
    /**
     * [actions description]
     * @type {Object}
     */
    [CALCULATE_PAYMENTS_TOTAL] ( state, payload){
        let currencyJournal = payload.paymentsCurrencyAmount.find(item => item.id ===payload.currency.id)
        let total = 0
        if (currencyJournal)
            total = currencyJournal.amount
        state.paymentsTotal = total
    },
    /**
     * [paidTotal description]
     * @type {[type]}
     */
    [SET_PAID_TOTAL] (state, payload) {
        state.paidTotal = payload
    },
    /**
     * [paymentsCurrencyJournal description]
     * @type {[type]}
     */
    [SET_PAYMENTS_CURRENCY_JOURNAL] ( state, payload ){
        state.paymentsCurrencyJournal = payload
    }
}

const actions = {
    /**
     * [INIT_PAYMENTS_JOURNALS]
     */
    [INIT_PAYMENTS_JOURNALS] ({ commit }, payload ) {
        commit(SET_PAYMENTS_JOURNALS, payload)
    },
    /**
     * [SELECT_PAYMENTS_JOURNALS]
     */
    [SELECT_PAYMENTS_JOURNALS] ({ commit, getters }, payload ) {
        commit(SELECTED_PAYMENTS_JOURNALS, payload)

        let currency = []
        if (payload.currency.id){
            currency = payload.currency
        } else {
            currency = getters.currencyCompany
        }
        commit(CALCULATE_PAYMENTS_TOTAL, {
            paymentsCurrencyAmount: getters.paymentsCurrencyAmount,
            currency: currency
        })

        commit(SET_PAYMENTS_CURRENCY_JOURNAL, currency)
    },
    /**
     * [paymentsCurrencyAmount description]
     * @type {Array}
     */
    [RESET_SELECTED_JOURNAL_PAYMENTS] ({ commit }) {
        commit(SELECTED_PAYMENTS_JOURNALS, [])
        commit(CALCULATE_PAYMENTS_TOTAL, {
            paymentsCurrencyAmount: [],
            currency: []
        })
        commit(SET_PAYMENTS_CURRENCY_JOURNAL, [])
    },
    /**
     * [ADD_PAYMENTS_MOVE_LINES]
     */
    [ADD_PAYMENTS_MOVE_LINES] ({ commit, getters, dispatch }, payload ) {
        commit(SET_ADD_PAYMENTS_MOVE_LINES, payload)
        commit(CALCULATE_CURRENCY_AMOUNT, {
            paymentsMoveLines: getters.paymentsMoveLines,
            currency: getters.currency
        })
        dispatch(RESET_SELECTED_JOURNAL_PAYMENTS, [])
    },
    /**
     * [REMOVE_PAYMENTS_MOVE_LINES]
     */
    [REMOVE_PAYMENTS_MOVE_LINES] ({ commit, getters, dispatch }, payload ) {
        commit(SET_REMOVE_PAYMENTS_MOVE_LINES, payload)
        commit(CALCULATE_CURRENCY_AMOUNT, {
            paymentsMoveLines: getters.paymentsMoveLines,
            currency: getters.currency
        })
        dispatch(RESET_SELECTED_JOURNAL_PAYMENTS, [])

    },
    /**
     * [CHANGE_INITIAL_PAID]
     */
    [CHANGE_INITIAL_PAID] ({ commit },payload) {
        commit(SET_PAID_TOTAL, payload)
    },
    /**
     * [RESET_SUPPLIER_PAYMENTS]
     */
    [RESET_SUPPLIER_PAYMENTS] ({ commit, getters }) {
        commit(SET_PAYMENTS_JOURNALS, [])
        commit(SELECTED_PAYMENTS_JOURNALS, [])
        commit(SET_REMOVE_PAYMENTS_MOVE_LINES, {
            mode: 'full',
            moveLines: getters.paymentsMoveLines
        })
        commit(CALCULATE_CURRENCY_AMOUNT, {
            currency: [],
            paymentsMoveLines: []
        })
        commit(SET_PAYMENTS_CURRENCY_JOURNAL, [])
        commit(CALCULATE_PAYMENTS_TOTAL, {
            paymentsCurrencyAmount: [],
            currency: []
        })
        commit(SET_PAID_TOTAL , 0)
    }
}

export default {
    state,
    getters,
    mutations,
    actions
}
